//css_ref System.Data;
//css_ref System.Transactions;
//css_ref System.Xml;
//css_ref Interop.SAPbobsCOM.dll;
//css_ref log4net.dll;
//css_ref Produmex.Foundation.dll;
//css_ref Produmex.Foundation.Data.dll;
//css_ref Produmex.Foundation.Data.Sbo.dll;
//css_ref Produmex.Foundation.Sbo.dll;
//css_ref Produmex.Sbo.Logex.Data.dll;
//css_ref Produmex.Sbo.Logex.Hitfar.dll;

using System;
using System.Diagnostics;
using System.Reflection;
using System.Transactions;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.Attributes;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Reflection;
using Produmex.Foundation.Resources;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;

namespace AddUpdateScript
{
	class Program
	{
		static string CONNECTION_STRING = "Server=localhost;CompanyDB=SBODemo_US;ExtraDb=PMX_SBODemo_US;UseTrusted=False;DbUserName=sa;DbPassword=B1Admin;UserName=manager;Password=produmex;Pooling=True;MinPoolSize=2;MaxPoolSize=10;ServerType=MSSQL2014;DBCultureInfo=en-US;LicenseServer=localhost:30000";

		/// <summary>
		/// Used for logging
		/// </summary>
		private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

		static void Main(string[] args)
		{
			try
			{
				Produmex.Foundation.Diagnostics.LogProvider.SetConsoleAppender("WARN");
				Execute();
			}
			catch (Exception ex)
			{
				s_log.Error("", ex);
			}
			if (Debugger.IsAttached)
			{
				Console.WriteLine("Press <Enter>");
				Console.ReadLine();
			}
		}

		private static readonly Assembly s_hitfarAssembly = typeof(Produmex.Sbo.Logex.Hitfar.SboCommands).Assembly;
		private static string LOCATION_SCRIPTS = "Produmex.Sbo.Logex.Hitfar.Scripts.";

		static void Execute()
		{
			using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
			{
				using (PmxDbConnectionSql conn = new PmxDbConnectionSql(CONNECTION_STRING))
				{
					conn.Open(); //Open connection to SAP
					Console.WriteLine("Connection is open to database");

				}

				//Complete transaction
				scope.Complete();
			}

			//PMX connection
			using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
			{
				using (PmxDbConnectionSql dbConn = new PmxDbConnectionSql(CONNECTION_STRING))
				{
					dbConn.Open();

					Console.WriteLine("Tables");
					Console.WriteLine("Views");

                    dbConn.DbTool.CreateUpdateFunction("PMX_FN_GetOpenSalesOrdersWithStockStatusHIT", PreprocessSqlScript(ResourceHelper.ReadResourceContentAsString(s_hitfarAssembly, LOCATION_SCRIPTS + "Functions.PMX_FN_GetOpenSalesOrdersWithStockStatusHIT.sql"), dbConn));
                    dbConn.DbTool.CreateUpdateView("PMX_OPEN_SALES_ORDERS_HIT", PreprocessSqlScript(ResourceHelper.ReadResourceContentAsString(s_hitfarAssembly, LOCATION_SCRIPTS + "Views.PMX_OPEN_SALES_ORDERS_HIT.sql"), dbConn));

                    //Complete transaction
                    scope.Complete();
				}
			}

			Console.WriteLine("Done ...");
		}

		/// <summary>
		/// Add/Update <see cref="PmxExtension"/>
		/// </summary>
		/// <param name="code">The code of the extension.</param>
		/// <param name="name">The name of the extension.</param>
		/// <param name="type">The type of the extension.</param>
		/// <param name="extensionTypeCode">The extension type code.</param>
		/// <param name="conn">The SBO connection</param>
		private static void AddUpdateExtension(string code, string name, Type type, string extensionTypeCode, PmxDbConnectionSql conn)
		{
			PmxExtensionProvider prov = new PmxExtensionProvider(conn);
			PmxExtension extension = null;
			try
			{
				bool newBO = false;
				extension = prov.GetBO(code);
				if (extension == null)
				{
					newBO = true;
					extension = prov.GetNewBO();
					extension.Code = code;
				}
				extension.Name = name;
				extension.ExtensionTypeCode = extensionTypeCode;
				extension.Type = ReflectionHelper.GetTypeAndAssemblyName(type);
				if (newBO)
				{
					prov.AddBO(extension);
				}
				else
				{
					prov.UpdateBO(extension, false, false);
				}
			}
			finally
			{
				if (extension != null) extension.Dispose();
			}
		}




		/// <summary>
		/// Add/Update script workflow
		/// </summary>
		/// <param name="executionType">Type of the execution.</param>
		/// <param name="name">The name of the flow.</param>
		/// <param name="sourceFilePath">Name of the file.</param>
		/// <param name="sqlConn">The SQL conn.</param>
		private static void AddUpdateWorkflowScript(string executionType, string name, string sourceFilePath, PmxDbConnectionSql sqlConn)
		{
			PmxWorkflowProvider wfProvider = new PmxWorkflowProvider(sqlConn);
			PmxWorkflow workflow = wfProvider.GetWorkflowByName(name);
			PmxWorkflowScriptProvider wfscProvider = new PmxWorkflowScriptProvider(sqlConn);
			PmxWorkflowScript bo = null;
			if (workflow == null)
			{
				//add the workflow
				bo = wfscProvider.GetNewBO();
				bo.Name = name;
				wfscProvider.AddBO(bo);
			}
			else
			{
				if (workflow.ObjType == Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxObjectTypes.PmxWorkflowScript)
				{
					bo = wfscProvider.GetBO(workflow.InternalKey);

				}
			}

			if (bo != null)
			{
				bo.ExecutionType = executionType;

				wfscProvider.UpdateSourceFromFileAndCompile(bo, sourceFilePath, sqlConn);
			}



		}

		/// <summary>
		/// Runs the table SQL script.
		/// </summary>
		/// <param name="tableScript">The table script.</param>
		/// <param name="conn">The conn.</param>
		private static void RunTableSqlScript(string tableScript, PmxDbConnection conn)
		{
			string content = ResourceHelper.ReadResourceContentAsString(Assembly.GetAssembly(typeof(Produmex.Sbo.Logex.Hitfar.SboCommands)), "Produmex.Sbo.Logex.Hitfar.Scripts.Tables." + tableScript);
			SboRecordsetHelper.RunSqlScript(s_log, content, conn);
		}

		/// <summary>
		/// Adds the update user fields not in transaction.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="types">The types.</param>
		private static void AddUpdateUserFieldsNotInTransaction(SboConnection conn, params Type[] types)
		{
			SboUserFieldMDProvider userFieldMDProv = new SboUserFieldMDProvider(conn);
			//System.Data.Common.DbTransaction transaction = null;
			try
			{
				//transaction = this.Connection.BeginTransaction();
				userFieldMDProv.AddUpdateUserFields(types);

				//Commit transaction
				//transaction.Commit();
			}
			catch (Exception)
			{
				//if (transaction != null) transaction.Rollback();
				throw;
			}
		}

		/// <summary>
		/// Runs the stored procedure SQL script.
		/// </summary>
		/// <param name="spScript">The sp script.</param>
		/// <param name="conn">The conn.</param>
		private static void RunStoredProcedureSqlScript(string spScript, PmxDbConnection conn)
		{
			string content = ResourceHelper.ReadResourceContentAsString(Assembly.GetAssembly(typeof(Produmex.Sbo.Logex.Hitfar.SboCommands)), "Produmex.Sbo.Logex.Hitfar.Scripts.StoredProcedures." + spScript);
			SboRecordsetHelper.RunSqlScript(s_log, content, conn);
		}

        /// <summary>
        /// Preprocesses the SQL script.
        /// </summary>
        /// <param name="query">The query</param>
        /// <param name="conn">The connection</param>
        /// <returns></returns>
        private static string PreprocessSqlScript(string query, PmxDbConnection conn)
        {
            string serverType;
            switch (conn.ConnectionStringObject.ServerType)
            {
                case SboServerType.HANA:
                    serverType = "HANA";
                    query = query.Replace("{DB-SCHEMA}", "\"" + conn.ConnectionStringObject.CompanyDBSchema.SchemaName + "\"");
                    break;
                default:
                    serverType = "MSSQL";
                    break;
            }

            int mssqlBlockBegin = query.IndexOf("--<?DB:");
            while (mssqlBlockBegin >= 0)
            {
                int mssqlBlockEnd = query.IndexOf("--?>", mssqlBlockBegin);
                if (mssqlBlockEnd < 0)
                {
                    break;
                }

                if (query.Substring(mssqlBlockBegin + "--<?DB:".Length, serverType.Length) == serverType)
                {
                    query = query.Substring(0, mssqlBlockBegin)
                          + query.Substring(mssqlBlockBegin + "--<?DB:".Length + serverType.Length, mssqlBlockEnd - mssqlBlockBegin - "--<?DB:".Length - serverType.Length)
                          + query.Substring(mssqlBlockEnd + "--?>".Length);
                    mssqlBlockEnd -= "--<?DB:".Length + serverType.Length + "--?>".Length;
                }
                else
                {
                    query = query.Substring(0, mssqlBlockBegin) + query.Substring(mssqlBlockEnd + "--?>".Length);
                    mssqlBlockEnd = mssqlBlockBegin;
                }
                mssqlBlockBegin = query.IndexOf("--<?DB:", mssqlBlockEnd);
            }

            if (conn.ConnectionStringObject.ServerType == SboServerType.HANA)
            {
                int tokenPos = query.IndexOf("ISNULL(", StringComparison.OrdinalIgnoreCase);
                while (tokenPos >= 0)
                {
                    int endPos = tokenPos + "ISNULL(".Length;
                    query = query.Substring(0, tokenPos) + "IFNULL(" + query.Substring(endPos);
                    tokenPos = query.IndexOf("ISNULL(", endPos, StringComparison.OrdinalIgnoreCase);
                }

                tokenPos = query.IndexOf("@");
                while (tokenPos >= 0)
                {
                    if (query[tokenPos - 1] != '"' && Char.IsLetter(query[tokenPos + 1]))
                    {
                        query = query.Substring(0, tokenPos) + "p_" + query.Substring(tokenPos + 1);
                    }
                    tokenPos = query.IndexOf("@", tokenPos + 2);
                }
            }

            return query;
        }
    }
}
